@@ -10,10 +10,11 @@ from gi.repository import GdkPixbuf
 from gi.repository import Gtk
 
 from kuwo import Config
+_ = Config._
 from kuwo import Net
 from kuwo import Widgets
+from kuwo.log import logger
 
-_ = Config._
 
 class TopCategories(Gtk.Box):
     '''Categories tab in notebook.'''
@@ -53,31 +54,31 @@ class TopCategories(Gtk.Box):
         self.scrolled_main = Gtk.ScrolledWindow()
         self.pack_start(self.scrolled_main, True, True, 0)
         # logo, name, nid, num of lists(info), tooltip
-        self.liststore_main = Gtk.ListStore(
-                GdkPixbuf.Pixbuf, str, int, str, str)
+        self.liststore_main = Gtk.ListStore(GdkPixbuf.Pixbuf, str, int,
+                                            str, str)
         iconview_main = Widgets.IconView(self.liststore_main, tooltip=4)
-        iconview_main.connect(
-                'item_activated', self.on_iconview_main_item_activated)
+        iconview_main.connect('item_activated',
+                              self.on_iconview_main_item_activated)
         self.scrolled_main.add(iconview_main)
 
         self.scrolled_sub1 = Gtk.ScrolledWindow()
         self.pack_start(self.scrolled_sub1, True, True, 0)
         # logo, name, nid, num of lists(info), tooltip
-        self.liststore_sub1 = Gtk.ListStore(
-                GdkPixbuf.Pixbuf, str, int, str, str)
+        self.liststore_sub1 = Gtk.ListStore(GdkPixbuf.Pixbuf, str, int,
+                                            str, str)
         iconview_sub1 = Widgets.IconView(self.liststore_sub1, tooltip=4)
-        iconview_sub1.connect(
-                'item_activated', self.on_iconview_sub1_item_activated)
+        iconview_sub1.connect('item_activated',
+                              self.on_iconview_sub1_item_activated)
         self.scrolled_sub1.add(iconview_sub1)
 
         self.scrolled_sub2 = Gtk.ScrolledWindow()
         self.pack_start(self.scrolled_sub2, True, True, 0)
         # logo, name, nid, info, tooltip
-        self.liststore_sub2 = Gtk.ListStore(
-                GdkPixbuf.Pixbuf, str, int, str, str)
+        self.liststore_sub2 = Gtk.ListStore(GdkPixbuf.Pixbuf, str, int,
+                                            str, str)
         iconview_sub2 = Widgets.IconView(self.liststore_sub2, tooltip=4)
-        iconview_sub2.connect(
-                'item_activated', self.on_iconview_sub2_item_activated)
+        iconview_sub2.connect('item_activated',
+                              self.on_iconview_sub2_item_activated)
         self.scrolled_sub2.add(iconview_sub2)
 
         self.scrolled_songs = Gtk.ScrolledWindow()
@@ -90,30 +91,34 @@ class TopCategories(Gtk.Box):
         self.scrolled_sub2.hide()
         self.scrolled_songs.hide()
 
+        def _on_get_nodes(info, error):
+            if not info or not info[0] or not info[1] or error:
+                logger.error('TopCategories._on_get_nodes(): %s, %s' %
+                             (info, error))
+                return
+            nodes, total_pages = info
+            urls = []
+            tree_iters = []
+            for node in nodes:
+                # skip 'xx专区'
+                if node['disname'].endswith('专区'):
+                    continue
+                tree_iter = self.liststore_main.append([
+                    self.app.theme['anonymous'],
+                    Widgets.unescape(node['disname']),
+                    int(node['id']),
+                    Widgets.unescape(node['info']),
+                    Widgets.set_tooltip(node['disname'], node['info']),
+                ])
+                tree_iters.append(tree_iter)
+                urls.append(node['pic'])
+            self.liststore_main.timestamp = time.time()
+            Net.async_call(Net.update_liststore_images, self.liststore_main, 0,
+                           tree_iters, urls)
+
         nid = 5
         page = 0
-        nodes, total_page = Net.get_nodes(nid, page)
-        if not nodes:
-            print('Failed to get nodes, do something!')
-            return
-        urls = []
-        tree_iters = []
-        for node in nodes:
-            # skip 'xx专区' categories
-            if node['disname'].endswith('专区'):
-                continue
-            tree_iter = self.liststore_main.append([
-                self.app.theme['anonymous'],
-                Widgets.unescape(node['disname']),
-                int(node['id']),
-                Widgets.unescape(node['info']),
-                Widgets.set_tooltip(node['disname'], node['info']),
-                ])
-            tree_iters.append(tree_iter)
-            urls.append(node['pic'])
-        self.liststore_main.timestamp = time.time()
-        Net.update_liststore_images(
-                self.liststore_main, 0, tree_iters, urls)
+        Net.async_call(Net.get_nodes, nid, page, callback=_on_get_nodes)
 
     def on_iconview_main_item_activated(self, iconview, path):
         model = iconview.get_model()
@@ -132,30 +137,31 @@ class TopCategories(Gtk.Box):
         self.show_sub1(init=True)
 
     def show_sub1(self, init=False):
-        def _show_sub1(sub1_args, error=None):
-            nodes, self.sub1_total = sub1_args
-            if error or not nodes or not self.sub1_total:
+        def _on_show_sub1(info, error=None):
+            if not info or not info[0] or not info[1] or error:
+                logger.error('show_sub1(): %s, %s' % (info, error))
                 return
+            nodes, self.sub1_total = info
             urls = []
             tree_iters = []
             for node in nodes:
-                _id = 'id' if self.use_sub2 else 'sourceid'
+                id_ = 'id' if self.use_sub2 else 'sourceid'
                 if 'tips' in node and len(node['tips']) > 5:
-                    tooltip = Widgets.set_tooltip_with_song_tips(
-                            node['name'], node['tips'])
+                    tooltip = Widgets.set_tooltip_with_song_tips(node['name'],
+                                                                 node['tips'])
                 else:
                     tooltip = Widgets.set_tooltip(node['name'], node['info'])
                 tree_iter = self.liststore_sub1.append([
                     self.app.theme['anonymous'],
                     Widgets.unescape(node['name']),
-                    int(node[_id]),
+                    int(node[id_]),
                     Widgets.unescape(node['info']),
                     tooltip,
-                    ])
+                ])
                 urls.append(node['pic'])
                 tree_iters.append(tree_iter)
-            Net.update_liststore_images(
-                    self.liststore_sub1, 0, tree_iters, urls)
+            Net.async_call(Net.update_liststore_images, self.liststore_sub1, 0,
+                           tree_iters, urls)
 
             self.sub1_page += 1
             if self.sub1_page < self.sub1_total - 1:
@@ -167,15 +173,14 @@ class TopCategories(Gtk.Box):
             self.button_sub1.hide()
             self.button_sub2.hide()
             self.control_box.hide()
-            self.scrolled_sub1.get_vadjustment().set_value(0)
+            self.scrolled_sub1.get_vscrollbar().set_value(0)
             self.scrolled_sub1.show_all()
             self.sub1_page = 0
             self.liststore_sub1.clear()
         if init or not hasattr(self.liststore_sub1, 'timestamp'):
             self.liststore_sub1.timestamp = time.time()
-        Net.async_call(
-                Net.get_nodes, _show_sub1, self.curr_sub1_id,
-                self.sub1_page)
+        Net.async_call(Net.get_nodes, self.curr_sub1_id, self.sub1_page,
+                       callback=_on_show_sub1)
 
     def on_iconview_sub1_item_activated(self, iconview, path):
         model = iconview.get_model()
@@ -194,9 +199,11 @@ class TopCategories(Gtk.Box):
             self.append_songs(init=True)
 
     def show_sub2(self, init=False):
-        def _show_sub2(sub2_args, error=None):
+        def _on_show_sub2(sub2_args, error=None):
             nodes, self.sub2_total = sub2_args
             if error or not nodes or not self.sub2_total:
+                logger.error('nodes: %s, self.sub2_total: %s, error: %s' %
+                        (nodes, self.sub2_total, error))
                 return
             urls = []
             tree_iters = []
@@ -206,13 +213,13 @@ class TopCategories(Gtk.Box):
                     Widgets.unescape(node['name']),
                     int(node['sourceid']),
                     Widgets.unescape(node['info']),
-                    Widgets.set_tooltip_with_song_tips(
-                        node['name'], node['tips']),
-                    ])
+                    Widgets.set_tooltip_with_song_tips(node['name'],
+                                                       node['tips'])
+                ])
                 urls.append(node['pic'])
                 tree_iters.append(tree_iter)
-            Net.update_liststore_images(
-                    self.liststore_sub2, 0, tree_iter, urls)
+            Net.async_call(Net.update_liststore_images, self.liststore_sub2, 0,
+                           tree_iters, urls)
 
             self.sub2_page += 1
             if self.sub2_page < self.sub2_total - 1:
@@ -221,15 +228,14 @@ class TopCategories(Gtk.Box):
         if init:
             self.scrolled_sub1.hide()
             self.button_sub1.show_all()
-            self.scrolled_sub2.get_vadjustment().set_value(0)
+            self.scrolled_sub2.get_vscrollbar().set_value(0)
             self.scrolled_sub2.show_all()
             self.sub2_page = 0
             self.liststore_sub2.clear()
         if init or not hasattr(self.liststore_sub2, 'timestamp'):
             self.liststore_sub2.timestamp = time.time()
-        Net.async_call(
-                Net.get_nodes, _show_sub2, self.curr_sub2_id,
-                self.sub2_page)
+        Net.async_call(Net.get_nodes, self.curr_sub2_id, self.sub2_page,
+                       callback=_on_show_sub2)
 
     def on_iconview_sub2_item_activated(self, iconview, path):
         model = iconview.get_model()
@@ -240,25 +246,33 @@ class TopCategories(Gtk.Box):
         self.append_songs(init=True)
 
     def append_songs(self, init=False):
-        def _append_songs(songs_args, error=None):
-            songs, self.songs_total = songs_args
-            if not songs or self.songs_total == 0 or self.use_album:
-                songs = Net.get_album(self.curr_list_id)
-                self.songs_total = 1
-                if not songs:
-                    return
-                for song in songs:
-                    self.liststore_songs.append([
-                        True,
-                        Widgets.unescape(song['name']), 
-                        Widgets.unescape(song['artist']),
-                        Widgets.unescape(self.curr_list_name), 
-                        int(song['id']),
-                        int(song['artistid']), 
-                        int(self.curr_list_id),
-                        song['formats'],
-                    ])
+        def _on_get_album(songs, error=None):
+            self.songs_total = 1
+            if not songs or error:
+                logger.error('TopCategories._on_get_album: %s, %s' %
+                             (songs, error))
                 return
+            for song in songs:
+                self.liststore_songs.append([
+                    True,
+                    Widgets.unescape(song['name']),
+                    Widgets.unescape(song['artist']),
+                    Widgets.unescape(self.curr_list_name),
+                    int(song['id']),
+                    int(song['artistid']),
+                    int(self.curr_list_id),
+                    song['formats'],
+                ])
+
+        def _on_append_songs(info, error=None):
+            if not info or error:
+                logger.error('TopCategories.append_songs(): %s, %s' %
+                             (info, error))
+                return
+            songs, self.songs_total = info
+            if not songs or self.songs_total == 0 or self.use_album:
+                Net.async_call(Net.get_album, self.curr_list_id,
+                               callback=_on_get_album)
 
             for song in songs:
                 self.liststore_songs.append([
@@ -284,13 +298,12 @@ class TopCategories(Gtk.Box):
             if self.use_sub2:
                 self.scrolled_sub2.hide()
                 self.button_sub2.show_all()
-            self.scrolled_songs.get_vadjustment().set_value(0.0)
+            self.scrolled_songs.get_vscrollbar().set_value(0.0)
             self.scrolled_songs.show_all()
             self.liststore_songs.clear()
 
-        Net.async_call(
-                Net.get_themes_songs, _append_songs, self.curr_list_id,
-                self.songs_page)
+        Net.async_call(Net.get_themes_songs, self.curr_list_id,
+                       self.songs_page, callback=_on_append_songs)
 
     # buttonbox
     def on_button_main_clicked(self, btn):
