@@ -7,15 +7,16 @@
 import html
 from html.parser import HTMLParser
 import os
+import traceback
+
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import GLib
 from gi.repository import Gtk
 
-
 from kuwo import Config
-
 _ = Config._
+from kuwo.log import logger
 
 def unescape(tooltip):
     html_parser = HTMLParser()
@@ -36,8 +37,8 @@ def short_tooltip(tooltip, length=10):
     return short_str(escape(tooltip), length)
 
 def set_tooltip(head, body=''):
-    return '<b>{0}</b>\n\n{1}'.format(
-            escape(unescape(head)), escape(unescape(body)))
+    return '<b>{0}</b>\n\n{1}'.format(escape(unescape(head)),
+                                      escape(unescape(body)))
 
 def set_tooltip_with_song_tips(head, tip):
     songs = tip.split(';')
@@ -49,8 +50,8 @@ def set_tooltip_with_song_tips(head, tip):
         item = song.split('@')
         try:
             results.append(fmt.format(escape(item[1]), escape(item[3])))
-        except IndexError as e:
-            continue
+        except IndexError:
+            logger.error(traceback.format_exc())
     return '<b>{0}</b>\n\n{1}'.format(escape(head), '\n'.join(results))
 
 def song_row_to_dict(song_row, start=1):
@@ -114,7 +115,6 @@ class ListRadioButton(Gtk.RadioButton):
         self.props.draw_indicator = False
         if last_button:
             self.join_group(last_button)
-        # it might need a class name.
 
 
 class BoldLabel(Gtk.Label):
@@ -161,7 +161,7 @@ class FolderChooser(Gtk.Box):
         dialog = Gtk.FileChooserDialog(_('Choose a Folder'), self.parent,
                 Gtk.FileChooserAction.SELECT_FOLDER,
                 (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                    Gtk.STOCK_OK, Gtk.ResponseType.OK))
+                Gtk.STOCK_OK, Gtk.ResponseType.OK))
 
         dialog.connect('file-activated', on_dialog_file_activated)
         response = dialog.run()
@@ -170,14 +170,11 @@ class FolderChooser(Gtk.Box):
             return
         dialog.destroy()
 
+
 class TreeViewColumnText(Gtk.TreeViewColumn):
 
     def __init__(self, *args, **keys):
         super().__init__(*args, **keys)
-        # This is the best option, but Gtk raises some Exceptions like:
-        # (kuwo.py:14225): Gtk-CRITICAL **: _gtk_tree_view_column_autosize: assertion `GTK_IS_TREE_VIEW (tree_view)' failed
-        # I don't know why that happens and how to fix it.  
-        #self.props.sizing = Gtk.TreeViewColumnSizing.AUTOSIZE
         self.props.sizing = Gtk.TreeViewColumnSizing.GROW_ONLY
         self.props.expand = True
         self.props.max_width = 280
@@ -364,20 +361,18 @@ class TreeViewSongs(Gtk.TreeView):
             self.app.playlist.play_song(song)
         elif index == self.COL_ARTIST:
             if not song['artist']:
-                print('artist is empty, no searching')
                 return
             self.app.search.search_artist(song['artist'])
         elif index == self.COL_ALBUM:
             if not song['album']:
-                print('album is empty, no searching')
                 return
             self.app.search.search_album(song['album'])
 
 
 def network_error(parent, msg):
-    dialog = Gtk.MessageDialog(
-            parent, Gtk.DialogFlags.MODAL,
-            Gtk.MessageType.ERROR, Gtk.ButtonsType.CLOSE, msg)
+    dialog = Gtk.MessageDialog(parent, Gtk.DialogFlags.MODAL,
+                               Gtk.MessageType.ERROR,
+                               Gtk.ButtonsType.CLOSE, msg)
     dialog.format_secondary_text(
             _('Please check network connection and try again'))
     dialog.run()
@@ -386,8 +381,8 @@ def network_error(parent, msg):
 def filesystem_error(parent, path):
     msg = _('Failed to open file or direcotry')
     dialog = Gtk.MessageDialog(parent, Gtk.DialogFlags.MODAL,
-            Gtk.MessageType.ERROR, Gtk.ButtonsType.CLOSE, msg)
-    dialog.format_secondary_text(
-            _('Unable to access {0}').format(path))
+                               Gtk.MessageType.ERROR,
+                               Gtk.ButtonsType.CLOSE, msg)
+    dialog.format_secondary_text(_('Unable to access {0}').format(path))
     dialog.run()
     dialog.destroy()
